package com.yeejoin.amos.api.tool.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.api.component.face.model.AdapterCategoryModel;
import com.yeejoin.amos.api.component.face.service.AdapterCategoryService;
import com.yeejoin.amos.api.tool.face.model.VisualDesignerModel;
import com.yeejoin.amos.api.tool.face.service.VisualDesignerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.restful.doc.TycloudResource;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;
import java.util.List;


@RestController
@Api(tags = "平台通用分类")
@RequestMapping(value = "/adapter")
@TycloudResource(module = "atl", value = "adapter")
public class AdapterCategoryController {

    @Autowired
    private AdapterCategoryService adapterCategoryService;

    @Autowired
    private VisualDesignerService visualDesignerService;

    /**
     * 新增
     * @param adapterCategoryModel AdapterCategory平台通用分类
     * @return AdapterCategoryModel
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/save")
    @ApiOperation(httpMethod = "POST", value = "新增", notes = "新增")
    public ResponseModel<AdapterCategoryModel> save(@RequestBody AdapterCategoryModel adapterCategoryModel) throws Exception {
        adapterCategoryModel = adapterCategoryService.createAdapterCategory(adapterCategoryModel);
        return ResponseHelper.buildResponse(adapterCategoryModel);
    }

    /**
     * 根据sequenceNbr更新
     * @param sequenceNbr 编码
     * @return AdapterCategoryModel
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PutMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "PUT", value = "根据sequenceNbr更新", notes = "根据sequenceNbr更新")
    public ResponseModel<AdapterCategoryModel> update(@RequestBody AdapterCategoryModel adapterCategoryModel,
                                                                   @PathVariable(value = "sequenceNbr") Long sequenceNbr) throws Exception {
        adapterCategoryModel.setSequenceNbr(sequenceNbr);
        return ResponseHelper.buildResponse(adapterCategoryService.updateAdapterCategory(adapterCategoryModel));
    }

    /**
     * 根据sequenceNbr删除
     * @param sequenceNbr 编码
     * @return Boolean
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @DeleteMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "DELETE", value = "根据sequenceNbr删除", notes = "根据sequenceNbr删除")
    public ResponseModel<Boolean> deleteBySequenceNbr(@PathVariable(value = "sequenceNbr") Long sequenceNbr) {
        return ResponseHelper.buildResponse(adapterCategoryService.deleteBySeq(sequenceNbr));
    }

    /**
     * 根据sequenceNbr查询
     * @param sequenceNbr 物理主键
     * @return ApiSourceDto
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "GET", value = "根据sequenceNbr查询单个", notes = "根据sequenceNbr查询单个")
    public  ResponseModel<AdapterCategoryModel> selectOne(@PathVariable Long sequenceNbr){
        return ResponseHelper.buildResponse(adapterCategoryService.queryBySeq(sequenceNbr));
    }

    /**
     * 列表树形结构查询
     * @return List<AdapterCategoryModel>
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @ApiOperation(httpMethod = "GET", value = "列表树形结构查询", notes = "列表树形结构查询")
    @GetMapping(value = "/tree")
    public ResponseModel<List<AdapterCategoryModel>> selectForTree(@RequestParam("categoryType") String categoryType) throws Exception {
        if ("TEMPLATE".equals(categoryType)){
            List<VisualDesignerModel> visualDesignerList = visualDesignerService.queryForVisualDesignerList();
            return null;
        }
        return ResponseHelper.buildResponse(adapterCategoryService.queryAdapterCategoryForTree(false,categoryType,""));
    }

}
