package com.yeejoin.amos.api.tool.controller;

import com.yeejoin.amos.api.component.face.model.AdapterCategoryModel;
import com.yeejoin.amos.api.tool.face.service.ApiSourceService;
import org.springframework.web.bind.annotation.RequestMapping;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.Api;
import org.springframework.web.bind.annotation.RestController;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.foundation.enumeration.UserType;

import java.util.List;

@CrossOrigin
@RestController
@Api(tags = "接口来源管理Api")
@RequestMapping(value = "/api_source")
public class ApiSourceResource {

    @Autowired
    private ApiSourceService apiSourceService;

    /**
     * 新增
     * @param model API来源信息
     * @return ApiSourceDto
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PostMapping(value = "/save")
    @ApiOperation(httpMethod = "POST", value = "新增", notes = "新增")
    public ResponseModel<AdapterCategoryModel> save(@RequestBody AdapterCategoryModel model) {
        model = apiSourceService.createByModel(model);
        return ResponseHelper.buildResponse(model);
    }

    /**
     * 根据sequenceNbr更新
     * @param model API来源信息
     * @param sequenceNbr 编码
     * @return ApiSourceDto
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @PutMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "PUT", value = "根据sequenceNbr更新", notes = "根据sequenceNbr更新")
    public ResponseModel<AdapterCategoryModel> updateBySeqApiSource(
            @RequestBody AdapterCategoryModel model,
            @PathVariable(value = "sequenceNbr") Long sequenceNbr) {
        model.setSequenceNbr(sequenceNbr);
        return ResponseHelper.buildResponse(apiSourceService.updateByModel(model));
    }

    /**
     * 根据sequenceNbr删除
     * @param sequenceNbr 编码
     * @return Boolean
     * 这里做单个删除
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @DeleteMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "DELETE", value = "根据sequenceNbr删除", notes = "根据sequenceNbr删除")
    public ResponseModel<Boolean> deleteBySequenceNbr(@PathVariable(value = "sequenceNbr") Long sequenceNbr) {
        return ResponseHelper.buildResponse(apiSourceService.deleteById(sequenceNbr));
    }

    /**
     * 根据sequenceNbr查询
     * @param sequenceNbr 物理主键
     * @return ApiSourceDto
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @GetMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "GET", value = "根据sequenceNbr查询单个", notes = "根据sequenceNbr查询单个")
    public ResponseModel<AdapterCategoryModel> selectOne(@PathVariable Long sequenceNbr) {
        return ResponseHelper.buildResponse(apiSourceService.queryBySequenceNbr(sequenceNbr));
    }

    /**
     * 列表全部数据查询
     * @return List<ApiSourceDto>
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
    @ApiOperation(httpMethod = "GET", value = "列表全部数据查询", notes = "列表全部数据查询")
    @GetMapping(value = "/list")
    public ResponseModel<List<AdapterCategoryModel>> selectForList(@RequestParam("categoryType") String categoryType) {
        return ResponseHelper.buildResponse(apiSourceService.queryForApiSourceLists(categoryType));
    }

}
