package com.yeejoin.amos.api.tool.face.service;

import com.yeejoin.amos.api.tool.face.orm.entity.ApiInfo;
import com.yeejoin.amos.api.tool.face.orm.dao.ApiInfoMapper;
import com.yeejoin.amos.api.tool.face.model.ApiInfoModel;
import com.yeejoin.amos.api.tool.face.rowmapper.StudioDataApiRowMapper;
import com.yeejoin.amos.api.tool.utils.ToolUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.typroject.tyboot.core.foundation.context.RequestContext;
import org.typroject.tyboot.core.rdbms.service.BaseService;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import java.util.*;

@Service
public class ApiInfoService extends BaseService<ApiInfoModel,ApiInfo,ApiInfoMapper> {

    public ApiInfoModel insertModel(ApiInfoModel apiInfoModel){
        JdbcTemplate jdbcTemplate= ToolUtils.getJdbcTemplate();
        Random random = new Random();
        Long primaryKey = random.nextLong();
        String loginId = RequestContext.getLoginId();
        String sql="INSERT INTO studio_data_api (SEQUENCE_NBR,API_SOURCE,API_NAME,HTTP_METHOD,API_PATH,RES_TYPE,BODY_VALUE,RULE_DATA,REC_USER_ID,REC_DATE)" +
                " VALUES(?,?,?,?,?,?,?,?,?,?)";
        Object [] params={primaryKey,apiInfoModel.getApiSource(),apiInfoModel.getApiName(),apiInfoModel.getHttpMethod(),
                apiInfoModel.getApiPath(),apiInfoModel.getResType(),apiInfoModel.getBodyValue(),
                apiInfoModel.getRuleData(),loginId,new Date()};
        jdbcTemplate.update(sql,params);
        return apiInfoModel;
    }

    /**
     * 更新操作
     */
    public ApiInfoModel updateByModel(ApiInfoModel apiInfoModel){
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="UPDATE studio_data_api SET API_SOURCE=?,API_NAME=?,HTTP_METHOD=?,API_PATH=?,RES_TYPE=?," +
                " BODY_VALUE=?,RULE_DATA=? WHERE SEQUENCE_NBR=?";
        Object[] params={apiInfoModel.getApiSource(),apiInfoModel.getApiName(),apiInfoModel.getHttpMethod(),
                apiInfoModel.getApiPath(),apiInfoModel.getResType(),apiInfoModel.getBodyValue(),apiInfoModel.getRuleData(),apiInfoModel.getSequenceNbr()};
        jdbcTemplate.update(sql,params);
        return apiInfoModel;
    }

    /**
     * 分页查询
     */
    public Page<ApiInfoModel> queryForApiInfoPage(Page<ApiInfoModel> page,String apiSource,String apiName){
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        Long curr=page.getCurrent();
        Long pageSize=page.getSize();
        int startIndex = (curr.intValue() - 1) * pageSize.intValue();
        // 查询总记录数
        String countSql = "SELECT COUNT(*) FROM studio_data_api WHERE API_SOURCE = ? AND API_NAME = ?";
        // 构建 SQL 查询语句
        String sql = "SELECT * FROM studio_data_api WHERE API_SOURCE = ? AND API_NAME = ? LIMIT ? , ?";
        Long total;
        Object[] params = {apiSource, apiName, startIndex, pageSize};
        if (apiName==null||"".equals(apiName)){
            countSql="SELECT COUNT(*) FROM studio_data_api WHERE API_SOURCE = ?";
            total=jdbcTemplate.queryForObject(countSql, Long.class, apiSource);
            sql="SELECT * FROM studio_data_api WHERE API_SOURCE = ? LIMIT ? , ?";
            // 构建参数数组
            params = new Object[]{apiSource,startIndex, pageSize};
        }else {
            total=jdbcTemplate.queryForObject(countSql, Long.class,apiSource,apiName);
        }
        // 查询分页数据
        List<ApiInfoModel> data = jdbcTemplate.query(sql,new StudioDataApiRowMapper(),params);
        // 构建Page对象
        Page<ApiInfoModel> pageResult = new Page<>();
        pageResult.setCurrent(pageSize);
        pageResult.setSize(pageSize);
        pageResult.setTotal(total);
        pageResult.setRecords(data);
        return pageResult;
    }

    /**
     * 单个查询
     */
    public ApiInfoModel selectOneApiInfo(Long sequenceNbr){
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="SELECT * FROM studio_data_api WHERE SEQUENCE_NBR=?";
        ApiInfoModel apiInfoModel = jdbcTemplate.queryForObject(sql,new Long[]{sequenceNbr},new StudioDataApiRowMapper());
        return apiInfoModel;
    }

    /**
     * 根据主键单个删除
     */
    public Boolean deleteById(Long sequenceNbr){
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="DELETE FROM STUDIO_DATA_API WHERE SEQUENCE_NBR=?";
        try {
            jdbcTemplate.update(sql,sequenceNbr);
            return true;
        }catch (Exception e){
            return false;
        }
    }

    /**
     * 根据主键批量删除
     */
    public Boolean deleteByIds(List<String> sequenceNbrs){
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
         String sql="DELETE FROM STUDIO_DATA_API WHERE SEQUENCE_NBR=?";
         try {
             sequenceNbrs.stream().forEach(sequenceNbr ->{
                 jdbcTemplate.update(sql,sequenceNbr);
             });
             return true;
         }catch (Exception e){
             return false;
         }
    }

    /**
     * 列表查询
     */
    public List<ApiInfoModel> queryForApiInfoList()   {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="SELECT * FROM studio_data_api";
        List<ApiInfoModel> apiInfoModels = jdbcTemplate.query(sql, new Object[]{}, new StudioDataApiRowMapper());
//      return this.queryForList("" , false);
        return apiInfoModels;
    }

    /**
     * 根据来源获取Api列表
     */
//    public List<ApiInfoModel> selectByApiSource(String apiSource)   {
//        return this.queryForList("" , false,apiSource);
//    }
//
//    /**
//     * 根据来源获取Api列表主键
//     */
//    public List<String> selectSeqByApiSource(String apiSource) {
//        List<String> sequenceNbrs = new ArrayList<>();
//        for(ApiInfoModel model : this.selectByApiSource(apiSource)){
//            sequenceNbrs.add(Long.toString(model.getSequenceNbr()));
//        }
//        return sequenceNbrs;
//    }

}
