package com.yeejoin.amos.api.tool.face.service;

import com.yeejoin.amos.api.component.face.model.AdapterCategoryModel;
import com.yeejoin.amos.api.tool.face.model.ApiSourceModel;
import com.yeejoin.amos.api.tool.face.orm.entity.ApiSource;
import com.yeejoin.amos.api.tool.face.orm.dao.ApiSourceMapper;
import com.yeejoin.amos.api.tool.face.rowmapper.ApiSourceRowMapper;
import com.yeejoin.amos.api.tool.utils.ToolUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.typroject.tyboot.core.foundation.context.RequestContext;
import org.typroject.tyboot.core.rdbms.service.BaseService;
import org.springframework.stereotype.Service;
import java.util.Date;
import java.util.List;
import java.util.Random;

@Service
public class ApiSourceService extends BaseService<ApiSourceModel, ApiSource, ApiSourceMapper> {

    public AdapterCategoryModel createByModel(AdapterCategoryModel model) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        Long sequenceNbr= new Random().nextLong();
        model.setSequenceNbr(sequenceNbr);
        String loginId = RequestContext.getLoginId();
        String agencyCode=RequestContext.getAgencyCode();
        Integer result = jdbcTemplate.queryForObject("SELECT COUNT(*) FROM adapter_category WHERE CATEGORY_TYPE=?", new String[]{"API"}, Integer.class);
        String sql="INSERT INTO adapter_category (SEQUENCE_NBR,CATEGORY_TYPE,PARENT_ID,CATEGORY_NAME,CATEGORY_KEY,SORT,AGENCY_CODE,IS_DELETE,REC_USER_ID,REC_DATE,CREATE_DATE) " +
                "VALUES (?,?,?,?,?,?,?,?,?,?,?)";
        jdbcTemplate.update(sql,new Object[]{model.getSequenceNbr(),"API",0l,model.getCategoryName(),
                model.getCategoryKey(),result+1,agencyCode==null?"":agencyCode,0,loginId,new Date(),new Date()});
        return model;
    }

    public List<AdapterCategoryModel> queryForApiSourceLists(String categoryType) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        List<AdapterCategoryModel> adapterCategoryModels = jdbcTemplate.query("SELECT * FROM adapter_category WHERE CATEGORY_TYPE=? AND IS_DELETE!=?",
                new Object[]{categoryType,ToolUtils.IS_DELETE},new ApiSourceRowMapper());
        return adapterCategoryModels;
    }

    public AdapterCategoryModel queryBySequenceNbr(Long sequenceNbr){
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="SELECT * FROM adapter_category WHERE SEQUENCE_NBR=? AND IS_DELETE=?";
        AdapterCategoryModel adapterCategoryModel = jdbcTemplate.queryForObject(sql,new Object[]{sequenceNbr,!ToolUtils.IS_DELETE},new ApiSourceRowMapper());
        return adapterCategoryModel;
    }

    /**
     * 列表查询
     */
//    public List<ApiSourceModel> queryForApiSourceList() {
//        return this.queryForList("", false);
//    }

    /**
     * 根据机构ID获取机构编码
     */
    public String queryCodeById(Long sequenceNbr){
        /**
         * 这里直接传入的主键查询表，因此不做CATEGORY_TYPE的分类
         */
        //ApiSourceModel model = this.queryModelByParams(sequenceNbr);
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="SELECT * FROM adapter_category WHERE SEQUENCE_NBR=? AND IS_DELETE!=?";
        AdapterCategoryModel model = jdbcTemplate.queryForObject(sql, new Object[]{sequenceNbr,ToolUtils.IS_DELETE},new ApiSourceRowMapper());
        return model.getCategoryKey()==null?"":model.getCategoryKey();
    }

    /**
     * 通过API来源ID删除API来源
     */
    public Boolean deleteById(Long sequenceNbr){
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String code = this.queryCodeById(sequenceNbr);
        /**
         * 这里是删除一级菜单，因为需要查询是否存在对应的二级菜单
         */
        String studioDataApiNum="SELECT COUNT(*) FROM studio_data_api WHERE API_SOURCE=?";
        Integer integer = jdbcTemplate.queryForObject(studioDataApiNum, new String[]{code}, Integer.class);
        if (integer!=null && integer>0){
            String realDelSql="DELETE FROM studio_data_api WHERE API_SOURCE=?";
            jdbcTemplate.update(realDelSql,new String[]{code});
        }
        String sql="UPDATE adapter_category SET IS_DELETE=? WHERE SEQUENCE_NBR=? ";
        try {
            jdbcTemplate.update(sql,new Object[]{ToolUtils.IS_DELETE,sequenceNbr});
            return true;
        }catch (Exception e){
            return false;
        }
    }

    public AdapterCategoryModel updateByModel(AdapterCategoryModel model) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="UPDATE adapter_category SET CATEGORY_NAME=?,CATEGORY_KEY=?,REC_DATE=? WHERE SEQUENCE_NBR=?";
        jdbcTemplate.update(sql,new Object[]{model.getCategoryName(),model.getCategoryKey(),new Date(),model.getSequenceNbr()});
        return model;
    }

}
