package com.yeejoin.amos.api.tool.face.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.yeejoin.amos.api.tool.face.model.TemplateCategoryModel;
import com.yeejoin.amos.api.tool.face.model.TemplateCategoryTreeModel;
import com.yeejoin.amos.api.tool.face.model.VisualDesignerModel;
import com.yeejoin.amos.api.tool.face.orm.dao.TemplateCategoryMapper;
import com.yeejoin.amos.api.tool.face.orm.entity.TemplateCategory;
import com.yeejoin.amos.api.tool.face.rowmapper.AdapterCategoryRowMapper;
import com.yeejoin.amos.api.tool.face.rowmapper.TemplateCategoryRowMapper;
import com.yeejoin.amos.api.tool.utils.ToolUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.typroject.tyboot.core.foundation.context.RequestContext;
import org.typroject.tyboot.core.rdbms.service.BaseService;
import org.springframework.stereotype.Service;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

@Service
public class TemplateCategoryService extends BaseService<TemplateCategoryModel, TemplateCategory, TemplateCategoryMapper> {

    @Autowired
    private VisualDesignerService visualDesignerService;

    private List<TemplateCategoryModel> templateCategoryModels=new ArrayList<>();
    /**
     * 列表查询
     */
    public List<TemplateCategoryModel> queryForTplCateList(String categoryType) {
//       jdbcTemplate = new JdbcTemplate(dataSource);
//       String sql = "SELECT * FROM adapter_category WHERE CATEGORY_TYPE=?";
//       List<TemplateCategoryModel> templateCategoryModels = jdbcTemplate.query(sql,new String[]{categoryType}, new TemplateCategoryRowMapper());
//       return this.queryForList("", false);
        return templateCategoryModels;
    }

    public TemplateCategoryModel queryWithSeq(Long sequenceNbr) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql = "SELECT * FROM adapter_category WHERE SEQUENCE_NBR=? AND IS_DELETE!=?";
        return jdbcTemplate.queryForObject(sql, new Object[]{sequenceNbr,ToolUtils.IS_DELETE},new TemplateCategoryRowMapper());
    }

    /**
     * 通过模板分类ID删除模板分类
     */
    public Boolean deleteById(Long sequenceNbr) {
        /**
         * 这里删除的是二级菜单，因此还需要查询是否存在三级菜单；如果有，则删除。
         */
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String studioTemplateNum = "SELECT COUNT(*) FROM studio_template WHERE CATEGORY_SEQ=? AND IS_DELETE!=?";
        Integer resultNum = jdbcTemplate.queryForObject(studioTemplateNum, new Object[]{sequenceNbr,ToolUtils.IS_DELETE}, Integer.class);
        try {
            if (resultNum != null && resultNum > 0) {
                String realDelSql = "UPDATE studio_template SET IS_DELETE=? WHERE CATEGORY_SEQ=?";
                jdbcTemplate.update(realDelSql, new Object[]{ToolUtils.IS_DELETE,sequenceNbr});
            }
            String sql = "UPDATE adapter_category SET IS_DELETE=? WHERE SEQUENCE_NBR=?";
            jdbcTemplate.update(sql, new Object[]{ToolUtils.IS_DELETE,sequenceNbr});
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 模板分类表单数据转化为树形节点
     */
    public List<TemplateCategoryTreeModel> tplCateToNode(List<TemplateCategoryModel> templateCategoryModels) {
        List<TemplateCategoryTreeModel> nodes = new ArrayList<>();
        for (TemplateCategoryModel model : templateCategoryModels) {
            TemplateCategoryTreeModel node = new TemplateCategoryTreeModel();
            node.setName(model.getCateName());
            node.setSequenceNbr(model.getSequenceNbr());
            node.setParentId(1L);
            node.setDesignerType(model.getDesignerType());
            nodes.add(node);
        }
        return nodes;
    }

    /**
     * 修改后
     */
    public List<TemplateCategoryTreeModel> getTree(String categoryType) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        /**
         * 该方法返回封装好的TemplateCategoryModel类型的数据
         */
        templateCategoryModels = jdbcTemplate.query("SELECT  * FROM adapter_category WHERE CATEGORY_TYPE=? AND IS_DELETE!=?",new Object[]{categoryType,ToolUtils.IS_DELETE}, new AdapterCategoryRowMapper());
//      List<TemplateCategoryModel> templateCategoryModels = this.queryForTplCateList();
        List<TemplateCategoryTreeModel> nodes = this.tplCateToNode(templateCategoryModels);
        List<VisualDesignerModel> visualDesignerLists = jdbcTemplate.query("SELECT * FROM studio_visual_designer", new Object[]{}, new BeanPropertyRowMapper(VisualDesignerModel.class));
//      List<VisualDesignerModel> visualDesignerLists = visualDesignerService.queryForVisualDesignerList();
        nodes.addAll(visualDesignerService.VisualDesignerToNode(visualDesignerLists));
        return getChildren(nodes, "", 0L);
    }


    /**
     * 获得模板分类子节点（递归）
     */
    public List<TemplateCategoryTreeModel> getChildren(List<TemplateCategoryTreeModel> nodes, String parent, Long id) {
        List<TemplateCategoryTreeModel> tree = new ArrayList<>();
        for (TemplateCategoryTreeModel node : nodes) {
            if (node.getParentId() == 0L && parent.equals("")) {
                node.setChildren(getChildren(nodes, node.getDesignerType(), node.getSequenceNbr()));
                tree.add(node);
            } else {
                if (parent.equals(node.getDesignerType()) && node.getParentId() != 0L) {
                    node.setParentId(id);
                    tree.add(node);
                }
            }
        }
        return tree;
    }

    public TemplateCategoryModel updateByModel(TemplateCategoryModel templateCategoryModel) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="UPDATE adapter_category SET CATEGORY_NAME=?,REC_DATE=?,EXTRAS=JSON_OBJECT('tempType', ?,'designerType',?) WHERE SEQUENCE_NBR=?";
        Object[] params={templateCategoryModel.getCateName()
                ,new Date(),templateCategoryModel.getTplType(),templateCategoryModel.getDesignerType(),templateCategoryModel.getSequenceNbr()};
        jdbcTemplate.update(sql,params);
        return templateCategoryModel;
    }

    public TemplateCategoryModel insertModel(TemplateCategoryModel templateCategoryModel) throws JsonProcessingException {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        Long parentId=visualDesignerService.selectSequenceNbr(templateCategoryModel.getDesignerType());
        Long sequenceNbr = new Random().nextLong();
        templateCategoryModel.setSequenceNbr(sequenceNbr);
        String loginId = RequestContext.getLoginId();
        String agencyCode=RequestContext.getAgencyCode();
        Integer result = jdbcTemplate.queryForObject("SELECT COUNT(*) FROM adapter_category WHERE CATEGORY_TYPE=?", new String[]{"TEMPLATE"}, Integer.class);
        String sql="INSERT INTO adapter_category (SEQUENCE_NBR,CATEGORY_TYPE,PARENT_ID,CATEGORY_NAME,SORT,AGENCY_CODE,IS_DELETE," +
                " REC_USER_ID,REC_DATE,CREATE_DATE,EXTRAS) VALUES (?,?,?,?,?,?,?,?,?,?,JSON_OBJECT('tempType', ?,'designerType',?))";
        Object[]params={sequenceNbr,"TEMPLATE",parentId,templateCategoryModel.getCateName(),result+1,agencyCode,0,loginId,new Date(),new Date(),templateCategoryModel.getTplType(),templateCategoryModel.getDesignerType()};
        jdbcTemplate.update(sql,params);
        return templateCategoryModel;
    }
}
