package com.yeejoin.amos.api.tool.face.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.api.tool.face.model.*;
import com.yeejoin.amos.api.tool.face.orm.dao.TemplateMapper;
import com.yeejoin.amos.api.tool.face.orm.entity.Template;
import com.yeejoin.amos.api.tool.face.rowmapper.StudioTemplateRowMapper;
import com.yeejoin.amos.api.tool.utils.ConvertUtil;
import com.yeejoin.amos.api.tool.utils.ToolUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.typroject.tyboot.core.foundation.context.RequestContext;
import org.typroject.tyboot.core.foundation.utils.ValidationUtil;
import org.typroject.tyboot.core.rdbms.service.BaseService;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.*;

@Service
public class TemplateService extends BaseService<TemplateModel, Template, TemplateMapper> {

    @Autowired
    private TemplateCategoryService templateCategoryService;


    /**
     * 分页查询
     */
    public Page<TemplateModel> queryForTplPage(Page<TemplateModel> page, Long categorySeq, String displayName,String categoryType) {
        List<TemplateCategoryTreeModel> modelList = templateCategoryService.getTree(categoryType);
        List<TemplateModel> records = new ArrayList<>();
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        for (TemplateCategoryTreeModel visualDesigner : modelList) {
            if (Objects.equals(categorySeq, visualDesigner.getSequenceNbr())) {
                for (TemplateCategoryTreeModel category : visualDesigner.getChildren()) {
                    List<TemplateModel> templateModels = jdbcTemplate.query("SELECT * FROM studio_template WHERE category_seq=? and IS_DELETE!=?",
                                new Object[]{category.getSequenceNbr(),ToolUtils.IS_DELETE},new StudioTemplateRowMapper());
//                  List<TemplateModel> templateModels = this.queryForTplListByCategorySeq(category.getSequenceNbr());
                    records.addAll(templateModels);
                }
                return this.queryForPages(records, page);
            }
        }
        Long curr=page.getCurrent();
        Long pageSize=page.getSize();
        int startIndex = (curr.intValue() - 1) * pageSize.intValue();
        // 查询总记录数
        String countSql = "SELECT COUNT(*) FROM studio_template WHERE category_seq = ? AND display_name = ? AND IS_DELETE!=?";
        // 构建 SQL 查询语句
        String sql = "SELECT * FROM studio_template WHERE category_seq = ? AND IS_DELETE!=? AND display_name = ? LIMIT ? , ? ";
        Long total;
        Object[] params = {categorySeq,ToolUtils.IS_DELETE,displayName, startIndex, pageSize};
        if (displayName==null||"".equals(displayName)){
            countSql="SELECT COUNT(*) FROM studio_template WHERE category_seq = ? AND IS_DELETE!=?";
            total=jdbcTemplate.queryForObject(countSql, Long.class, categorySeq,ToolUtils.IS_DELETE);
            sql="SELECT * FROM studio_template WHERE IS_DELETE!=? AND category_seq = ? LIMIT ? , ? ";
            // 构建参数数组
            params = new Object[]{ToolUtils.IS_DELETE,categorySeq,startIndex, pageSize};
        }else {
            total=jdbcTemplate.queryForObject(countSql, Long.class, categorySeq, displayName,1);
        }
        // 查询分页数据
        List<TemplateModel> data = jdbcTemplate.query(sql,new StudioTemplateRowMapper(),params);
        // 构建Page对象
        Page<TemplateModel> pageResult = new Page<>();
        pageResult.setCurrent(pageSize);
        pageResult.setSize(pageSize);
        pageResult.setTotal(total);
        pageResult.setRecords(data);
        return pageResult;
//      return this.queryForPage(page, null, false, categorySeq, displayName);
    }

    public Page<TemplateModel> queryForPages(List<TemplateModel> records, Page<TemplateModel> page) {
        Page<TemplateModel> entiryPage = new Page(page.getCurrent(), page.getSize());
        if (!ValidationUtil.isEmpty(records)) {
            page.setTotal(records.size());
            page.setPages(entiryPage.getPages());
            page.setCurrent(entiryPage.getCurrent());
            page.setSize(entiryPage.getSize());
            page.setRecords(records);
        }
        return page;
    }

    /**
     * 列表查询
     */
//    public List<TemplateModel> queryForTplListByCategorySeq(Long categorySeq) {
//        return this.queryForList("", false, categorySeq);
//    }

    /**
     * 根据模板分类ID获取模板sequenceNbrs
     */
//    public List<String> selectSeqsByCategorySeq(Long categorySeq) {
//        List<String> strings = new ArrayList<>();
//        for (TemplateModel model : this.queryForTplListByCategorySeq(categorySeq)) {
//            strings.add(Long.toString(model.getSequenceNbr()));
//        }
//        return strings;
//    }


    /**
     * 列表查询
     */
    public List<TemplateModel> queryForTplList() {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="SELECT * FROM studio_template WHERE IS_DELETE!=?";
        List<TemplateModel> templateModels = jdbcTemplate.query(sql, new Object[]{ToolUtils.IS_DELETE}, new StudioTemplateRowMapper());
        List<TemplateModel> templateModelLists=new ArrayList<>();
        for (TemplateModel templateModel:templateModels){
            TemplateVoModel templateVoModel = this.convertVoModel(templateModel);
            if (templateVoModel.getContent() != null) {
                templateVoModel.setContentStr(templateVoModel.getContent().toString());
            }
            templateModelLists.add(templateModel);
        }
        return templateModelLists;
    }

    /**
     * 根据sequenceNbr获取模板并转化为VO
     */
    public TemplateVoModel queryForTplVo(Long sequenceNbr) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="SELECT * FROM studio_template WHERE SEQUENCE_NBR=? AND IS_DELETE!=?";
        TemplateModel templateModel=jdbcTemplate.queryForObject(sql,new Object[]{sequenceNbr,ToolUtils.IS_DELETE},new StudioTemplateRowMapper());
//      TemplateModel templateModel = this.queryBySeq(sequenceNbr);
        TemplateVoModel templateVoModel = this.convertVoModel(templateModel);
        if (templateModel.getContent() != null) {
            templateVoModel.setContentStr(templateModel.getContent().toString());
        }
        return templateVoModel;
    }

    /**
     * TplVoModel转化TplModel
     */
    public TemplateModel convertModel(TemplateVoModel tplVoModel) throws NoSuchFieldException, IllegalAccessException {
        TemplateModel templateModel = new TemplateModel();
        BeanUtils.copyProperties(tplVoModel, templateModel, "thumbs", "sketches", "contentStr");
        //判定thumbs为空
        if (tplVoModel.getThumbs().size() != 0) {
            templateModel.setThumb(ConvertUtil.objectListToUrl(tplVoModel.getThumbs()));
        }
        //判定sketch为空
        if (tplVoModel.getSketches().size() != 0) {
            templateModel.setSketch(ConvertUtil.objectListToUrl(tplVoModel.getSketches()));
        }
        templateModel.setContent(JSON.parseObject(tplVoModel.getContentStr()));
        return templateModel;
    }

    /**
     * 读取json文件
     */
    public String readMultipartFile(MultipartFile multipartFile) throws IOException {
        Reader reader = new InputStreamReader(multipartFile.getInputStream(), "utf-8");
        int ch = 0;
        StringBuffer stringBuffer = new StringBuffer();
        while ((ch = reader.read()) != -1) {
            stringBuffer.append((char) ch);
        }
        reader.close();
        return stringBuffer.toString();
    }

    /**
     * 通过json文件建立空白模板
     */
    public TemplateModel fileToTpl(MultipartFile multipartFile) throws IOException {
        JSONObject content = JSON.parseObject(this.readMultipartFile(multipartFile));
        String str = "1668429235251548162";
        return new TemplateModel(content, Long.parseLong(str));
    }

    /**
     * model转化VO
     */
    public TemplateVoModel convertVoModel(TemplateModel templateModel) {
        TemplateVoModel tplVoModel = new TemplateVoModel();
        BeanUtils.copyProperties(templateModel, tplVoModel);
        if (templateModel.getSketch() != null)
            tplVoModel.setSketches(this.urlToPicModel(templateModel.getSketch()));
        if (templateModel.getThumb() != null)
            tplVoModel.setThumbs(this.urlToPicModel(templateModel.getThumb()));
        return tplVoModel;
    }

    /**
     * 生成随机UID
     */
    public String getUid() {
        UUID uuid = UUID.randomUUID();
        String str = uuid.toString();
        return str.replace("-", "");
    }

    /**
     * url转化图片
     */
    public List<PictureModel> urlToPicModel(String url) {
        List<PictureModel> pictureModels = new ArrayList<>();
        PictureModel pictureModel = new PictureModel("", this.getUid(), url, "done");
        pictureModels.add(pictureModel);
        return pictureModels;
    }

    public TemplateModel createByModel(TemplateModel templateModel) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        Long sequenceNbr = new Random().nextLong();
        String loginId = RequestContext.getLoginId();
        templateModel.setSequenceNbr(sequenceNbr);
        JSONObject content= templateModel.getContent();
        String sql="INSERT INTO studio_template (SEQUENCE_NBR,DISPLAY_NAME,COMPONENT_KEY,COMPONENT_NAME," +
                "CONTENT,CATEGORY_SEQ,THUMB,SKETCH,IS_DELETE,OPEN_STATUS,REC_DATE,REC_USER_ID) VALUES(?,?,?,?,?,?,?,?,?,?,?,?)";
        Object [] params={sequenceNbr,templateModel.getDisplayName(),templateModel.getComKey(),templateModel.getComName(),
                content==null?null:content.toString(),templateModel.getCateSeq(),templateModel.getThumb(),templateModel.getSketch(),
                templateModel.getIsDel(),templateModel.getOpenStatus(),new Date(),loginId};
        jdbcTemplate.update(sql,params);
        return templateModel;
    }

    public TemplateModel updateByModel(TemplateModel templateModel) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="UPDATE studio_template SET DISPLAY_NAME=?,COMPONENT_KEY=?,CATEGORY_SEQ=?,CONTENT=?,OPEN_STATUS=?," +
                " THUMB=?,SKETCH=?,COMPONENT_NAME=? WHERE SEQUENCE_NBR=?";
        Object[]params={templateModel.getDisplayName(),templateModel.getComKey(),templateModel.getCateSeq(),templateModel.getContent().toString(),
                templateModel.getOpenStatus(),templateModel.getThumb(),templateModel.getSketch(),templateModel.getComName(),templateModel.getSequenceNbr()};
        jdbcTemplate.update(sql,params);
        return templateModel;
    }

    public Boolean removeBySequenceNbr(Long sequenceNbr) {
        JdbcTemplate jdbcTemplate=ToolUtils.getJdbcTemplate();
        String sql="UPDATE studio_template SET IS_DELETE=? WHERE SEQUENCE_NBR=?";
        try {
            jdbcTemplate.update(sql,new Object[]{ToolUtils.IS_DELETE,sequenceNbr});
            return true;
        }catch (Exception e){
            e.printStackTrace();
            return false;
        }
    }

    public Boolean removeBySequenceNbrs(List<String> asList) {
        for (String sequenceNbr:asList) {
            removeBySequenceNbr(Long.valueOf(sequenceNbr));
        }
        return true;
    }
}
